Migrate Display Settings to Preferences.

Display settings can be represented with SwitchPreference and
SeekBarPreference. Note that SeekBarPreference looks slightly different
than what originally existed (seek bar is not aligned with text), but
this can be changed via changing the preference styles in the future.

Also, actual testing of the change of the display brightness was not
observed on the Mojave, because there isn't a correct LightsService
built to handle it. Still, all of the logic remains the same as before.

We include our own version of SeekBarPreference due to the inflexibility
of the standard SeekBarPreference. We add a functionality to send
continuous updates to the listener while changing the seek bar.

Bug: 117336555
Test: Build, Manual, Robolectric unit tests
Change-Id: I4c1f96fcfc414ca77c1addec64ab76a4c5e434d2
diff --git a/src/com/android/car/settings/display/AdaptiveBrightnessTogglePreferenceController.java b/src/com/android/car/settings/display/AdaptiveBrightnessTogglePreferenceController.java
new file mode 100644
index 0000000..215f851
--- /dev/null
+++ b/src/com/android/car/settings/display/AdaptiveBrightnessTogglePreferenceController.java
@@ -0,0 +1,75 @@
+/*
+ * Copyright (C) 2018 The Android Open Source Project
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
+package com.android.car.settings.display;
+
+import android.content.Context;
+import android.provider.Settings;
+
+import androidx.preference.Preference;
+import androidx.preference.TwoStatePreference;
+
+import com.android.car.settings.common.FragmentController;
+import com.android.car.settings.common.NoSetupPreferenceController;
+
+/** Business logic for controlling the adaptive brightness setting. */
+public class AdaptiveBrightnessTogglePreferenceController extends
+        NoSetupPreferenceController implements
+        Preference.OnPreferenceChangeListener {
+
+    public AdaptiveBrightnessTogglePreferenceController(Context context, String preferenceKey,
+            FragmentController fragmentController) {
+        super(context, preferenceKey, fragmentController);
+    }
+
+    @Override
+    public void updateState(Preference preference) {
+        if (!(preference instanceof TwoStatePreference)) {
+            throw new IllegalArgumentException("Expecting an instance of TwoStatePreference");
+        }
+        ((TwoStatePreference) preference).setChecked(isAdaptiveBrightnessChecked());
+    }
+
+    @Override
+    public int getAvailabilityStatus() {
+        return supportsAdaptiveBrightness() ? AVAILABLE : UNSUPPORTED_ON_DEVICE;
+    }
+
+    @Override
+    public boolean onPreferenceChange(Preference preference, Object newValue) {
+        if (!(preference instanceof TwoStatePreference)) {
+            throw new IllegalArgumentException("Expecting an instance of TwoStatePreference");
+        }
+        boolean enableAdaptiveBrightness = (boolean) newValue;
+        Settings.System.putInt(mContext.getContentResolver(),
+                Settings.System.SCREEN_BRIGHTNESS_MODE,
+                enableAdaptiveBrightness ? Settings.System.SCREEN_BRIGHTNESS_MODE_AUTOMATIC
+                        : Settings.System.SCREEN_BRIGHTNESS_MODE_MANUAL);
+        return true;
+    }
+
+    private boolean isAdaptiveBrightnessChecked() {
+        int brightnessMode = Settings.System.getInt(mContext.getContentResolver(),
+                Settings.System.SCREEN_BRIGHTNESS_MODE,
+                Settings.System.SCREEN_BRIGHTNESS_MODE_MANUAL);
+        return brightnessMode != Settings.System.SCREEN_BRIGHTNESS_MODE_MANUAL;
+    }
+
+    private boolean supportsAdaptiveBrightness() {
+        return mContext.getResources().getBoolean(
+                com.android.internal.R.bool.config_automatic_brightness_available);
+    }
+}
diff --git a/src/com/android/car/settings/display/BrightnessLevelPreferenceController.java b/src/com/android/car/settings/display/BrightnessLevelPreferenceController.java
new file mode 100644
index 0000000..4de0efc
--- /dev/null
+++ b/src/com/android/car/settings/display/BrightnessLevelPreferenceController.java
@@ -0,0 +1,95 @@
+/*
+ * Copyright (C) 2018 The Android Open Source Project
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
+package com.android.car.settings.display;
+
+import static com.android.settingslib.display.BrightnessUtils.GAMMA_SPACE_MAX;
+import static com.android.settingslib.display.BrightnessUtils.convertGammaToLinear;
+import static com.android.settingslib.display.BrightnessUtils.convertLinearToGamma;
+
+import android.car.userlib.CarUserManagerHelper;
+import android.content.Context;
+import android.os.PowerManager;
+import android.provider.Settings;
+
+import androidx.preference.Preference;
+import androidx.preference.PreferenceScreen;
+
+import com.android.car.settings.common.FragmentController;
+import com.android.car.settings.common.Logger;
+import com.android.car.settings.common.NoSetupPreferenceController;
+import com.android.car.settings.common.SeekBarPreference;
+
+/** Business logic for changing the brightness of the display. */
+public class BrightnessLevelPreferenceController extends NoSetupPreferenceController implements
+        Preference.OnPreferenceChangeListener {
+
+    private static final Logger LOG = new Logger(BrightnessLevelPreferenceController.class);
+    private final CarUserManagerHelper mCarUserManagerHelper;
+    private final int mMaximumBacklight;
+    private final int mMinimumBacklight;
+    private SeekBarPreference mPreference;
+
+    public BrightnessLevelPreferenceController(Context context, String preferenceKey,
+            FragmentController fragmentController) {
+        super(context, preferenceKey, fragmentController);
+        mCarUserManagerHelper = new CarUserManagerHelper(mContext);
+        PowerManager powerManager = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
+        mMaximumBacklight = powerManager.getMaximumScreenBrightnessSetting();
+        mMinimumBacklight = powerManager.getMinimumScreenBrightnessSetting();
+    }
+
+    @Override
+    public void displayPreference(PreferenceScreen screen) {
+        super.displayPreference(screen);
+        Preference preference = screen.findPreference(getPreferenceKey());
+        if (!(preference instanceof SeekBarPreference)) {
+            throw new IllegalStateException(
+                    "Preference for this controller should be SeekBarPreference");
+        }
+
+        mPreference = (SeekBarPreference) preference;
+        mPreference.setMax(GAMMA_SPACE_MAX);
+        mPreference.setValue(getSeekbarValue());
+        mPreference.setContinuousUpdate(true);
+    }
+
+    @Override
+    public boolean onPreferenceChange(Preference preference, Object newValue) {
+        if (!(preference instanceof SeekBarPreference)) {
+            throw new IllegalArgumentException("Expecting SeekBarPreference");
+        }
+        int gamma = (Integer) newValue;
+        int linear = convertGammaToLinear(gamma, mMinimumBacklight, mMaximumBacklight);
+        Settings.System.putIntForUser(mContext.getContentResolver(),
+                Settings.System.SCREEN_BRIGHTNESS, linear,
+                mCarUserManagerHelper.getCurrentProcessUserId());
+        return true;
+    }
+
+    private int getSeekbarValue() {
+        int gamma = GAMMA_SPACE_MAX;
+        try {
+            int linear = Settings.System.getIntForUser(mContext.getContentResolver(),
+                    Settings.System.SCREEN_BRIGHTNESS,
+                    mCarUserManagerHelper.getCurrentProcessUserId());
+            gamma = convertLinearToGamma(linear, mMinimumBacklight, mMaximumBacklight);
+        } catch (Settings.SettingNotFoundException e) {
+            LOG.w("Can't find setting for SCREEN_BRIGHTNESS.");
+        }
+        return gamma;
+    }
+}
diff --git a/src/com/android/car/settings/display/BrightnessListItem.java b/src/com/android/car/settings/display/BrightnessListItem.java
deleted file mode 100644
index e704895..0000000
--- a/src/com/android/car/settings/display/BrightnessListItem.java
+++ /dev/null
@@ -1,95 +0,0 @@
-/*
- * Copyright (C) 2018 The Android Open Source Project
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *      http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- */
-
-package com.android.car.settings.display;
-
-import static android.provider.Settings.System.SCREEN_BRIGHTNESS;
-
-import static com.android.settingslib.display.BrightnessUtils.GAMMA_SPACE_MAX;
-import static com.android.settingslib.display.BrightnessUtils.convertGammaToLinear;
-import static com.android.settingslib.display.BrightnessUtils.convertLinearToGamma;
-
-import android.car.userlib.CarUserManagerHelper;
-import android.content.Context;
-import android.os.PowerManager;
-import android.provider.Settings.SettingNotFoundException;
-import android.provider.Settings.System;
-import android.widget.SeekBar;
-import android.widget.SeekBar.OnSeekBarChangeListener;
-
-import androidx.car.widget.SeekbarListItem;
-
-import com.android.car.settings.R;
-import com.android.car.settings.common.Logger;
-
-/**
- * A ListItem that displays and sets display brightness.
- */
-public class BrightnessListItem extends SeekbarListItem {
-    private static final Logger LOG = new Logger(BrightnessListItem.class);
-    private CarUserManagerHelper mCarUserManagerHelper;
-    private final Context mContext;
-    private final int mMaximumBacklight;
-    private final int mMinimumBacklight;
-
-    /**
-     * Handles brightness change from user
-     */
-    private SeekBar.OnSeekBarChangeListener mOnSeekBarChangeListener =
-            new OnSeekBarChangeListener() {
-                @Override
-                public void onStartTrackingTouch(SeekBar seekBar) {
-                    // no-op
-                }
-
-                @Override
-                public void onStopTrackingTouch(SeekBar seekBar) {
-                    // no-op
-                }
-
-                @Override
-                public void onProgressChanged(SeekBar seekBar, int gamma, boolean fromUser) {
-                    int linear = convertGammaToLinear(gamma, mMinimumBacklight, mMaximumBacklight);
-                    System.putIntForUser(mContext.getContentResolver(), SCREEN_BRIGHTNESS, linear,
-                                         mCarUserManagerHelper.getCurrentForegroundUserId());
-                }
-            };
-
-    public BrightnessListItem(Context context) {
-        super(context);
-        mContext = context;
-        mCarUserManagerHelper = new CarUserManagerHelper(mContext);
-        PowerManager powerManager = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
-        mMaximumBacklight = powerManager.getMaximumScreenBrightnessSetting();
-        mMinimumBacklight = powerManager.getMinimumScreenBrightnessSetting();
-        setMax(GAMMA_SPACE_MAX);
-        setProgress(getSeekbarValue());
-        setOnSeekBarChangeListener(mOnSeekBarChangeListener);
-        setText(context.getString(R.string.brightness));
-    }
-
-    private int getSeekbarValue() {
-        int gamma = GAMMA_SPACE_MAX;
-        try {
-            int linear = System.getIntForUser(mContext.getContentResolver(), SCREEN_BRIGHTNESS,
-                                          mCarUserManagerHelper.getCurrentForegroundUserId());
-            gamma = convertLinearToGamma(linear, mMinimumBacklight, mMaximumBacklight);
-        } catch (SettingNotFoundException e) {
-            LOG.w("Can't find setting for SCREEN_BRIGHTNESS.");
-        }
-        return gamma;
-    }
-}
diff --git a/src/com/android/car/settings/display/DisplaySettingsFragment.java b/src/com/android/car/settings/display/DisplaySettingsFragment.java
index e7d6b83..03a45fb 100644
--- a/src/com/android/car/settings/display/DisplaySettingsFragment.java
+++ b/src/com/android/car/settings/display/DisplaySettingsFragment.java
@@ -15,71 +15,19 @@
  */
 package com.android.car.settings.display;
 
-import static android.provider.Settings.System.SCREEN_BRIGHTNESS_MODE;
-import static android.provider.Settings.System.SCREEN_BRIGHTNESS_MODE_AUTOMATIC;
-import static android.provider.Settings.System.SCREEN_BRIGHTNESS_MODE_MANUAL;
-
-import android.content.Context;
-import android.provider.Settings;
-
-import androidx.annotation.StringRes;
-import androidx.car.widget.ListItem;
-import androidx.car.widget.ListItemProvider;
-import androidx.car.widget.ListItemProvider.ListProvider;
-import androidx.car.widget.TextListItem;
+import androidx.annotation.XmlRes;
 
 import com.android.car.settings.R;
-import com.android.car.settings.common.ListItemSettingsFragment;
-
-import java.util.ArrayList;
-import java.util.List;
+import com.android.car.settings.common.BasePreferenceFragment;
 
 /**
  * Activity to host Display related preferences.
  */
-public class DisplaySettingsFragment extends ListItemSettingsFragment {
+public class DisplaySettingsFragment extends BasePreferenceFragment {
 
     @Override
-    @StringRes
-    protected int getTitleId() {
-        return R.string.display_settings;
-    }
-
-    @Override
-    public ListItemProvider getItemProvider() {
-        return new ListProvider(getListItems());
-    }
-
-    private List<ListItem> getListItems() {
-        List<ListItem> listItems = new ArrayList<>();
-        Context context = getContext();
-        if (supportsAdaptiveBrightness()) {
-            TextListItem adaptiveBrightnessItem = new TextListItem(context);
-            adaptiveBrightnessItem.setTitle(context.getString(R.string.auto_brightness_title));
-            adaptiveBrightnessItem.setBody(
-                    context.getString(R.string.auto_brightness_summary));
-            adaptiveBrightnessItem.setSwitch(
-                    isAdaptiveBrightnessChecked(),
-                    /* showDivider= */false,
-                    (button, isChecked) ->
-                            Settings.System.putInt(context.getContentResolver(),
-                                    SCREEN_BRIGHTNESS_MODE,
-                                    isChecked ? SCREEN_BRIGHTNESS_MODE_AUTOMATIC
-                                            : SCREEN_BRIGHTNESS_MODE_MANUAL));
-            listItems.add(adaptiveBrightnessItem);
-        }
-        listItems.add(new BrightnessListItem(context));
-        return listItems;
-    }
-
-    private boolean isAdaptiveBrightnessChecked() {
-        int brightnessMode = Settings.System.getInt(getContext().getContentResolver(),
-                SCREEN_BRIGHTNESS_MODE, SCREEN_BRIGHTNESS_MODE_MANUAL);
-        return brightnessMode != SCREEN_BRIGHTNESS_MODE_MANUAL;
-    }
-
-    private boolean supportsAdaptiveBrightness() {
-        return getContext().getResources().getBoolean(
-                com.android.internal.R.bool.config_automatic_brightness_available);
+    @XmlRes
+    protected int getPreferenceScreenResId() {
+        return R.xml.display_settings_fragment;
     }
 }
@@ -15,71 +15,19 @@
 
 /**
- * Activity to host Display related preferences.
+ * Preference fragment to host Display related preferences.
  */